/*
** Module   : Buffer.java
** Abstract : interface which must be extended by any business DMO interface
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 ECF 20130118 Created initial version.
** 002 VMN 20130207 Added support for handle-based methods:
**                  BUFFER-COPY (KW_BUF_COPY)
**                  BUFFER-COMPARE (KW_BUF_COMP)
**                  BUFFER-FIELD (KW_BUF_FLD)
**                  BUFFER-RELEASE (KW_BUF_REL)
**                  FIND-BY-ROWID (KW_FIND_BR)
**                  FIND-UNIQUE (KW_GET_UNI)
**                  FIND-FIRST (KW_FIND_1ST)
**                  FIND-LAST (KW_FIND_LST)
**                  and attributes:
**                  NUM-FIELDS (KW_NUM_FLD).
**                  INITIAL (KW_INIT)
**                  AMBIGUOUS (KW_AMBIG)
** 003 OM  20130214 Added bufferCreate and variants of bufferCopy.
** 004 SVL 20130213 Added disable*Triggers functions.
** 005 VMN 20130217 Extended from WriteableError and added support
**                  for handle-based attributes:
**                  CURRENT-ITERATION (KW_CUR_ITER)
**                  ERROR (KW_ERROR)
**                  DBNAME (KW_DBNAME)
**                  TABLE (KW_TABLE)
** 006 VMN 20130219 Added support for attributes
**                  TABLE-HANDLE (KW_TAB_HAND)
** 007 CA  20130221 Added support for UNIQUE-ID, ADM-DATA, NAMESPACE-URI attributes.
** 008 GES 20130224 Added support for a DELETE statement with a validation expression.
** 009 ECF 20130224 Added limited support for embedded SQL.
** 010 OM  20130221 Added Dereferenceable interface.
** 011 OM  20130301 Pushed up defaultBufferHandle() from TempTable.
** 012 VMN 20130305 Corrected support for handle-based methods:
**                  FIND-UNIQUE (KW_GET_UNI)
**                  FIND-FIRST (KW_FIND_1ST)
**                  FIND-LAST (KW_FIND_LST)
** 013 VMN 20130311 Corrected support for handle-based methods:
**                  FIND-UNIQUE (KW_GET_UNI)
**                  FIND-FIRST (KW_FIND_1ST)
**                  FIND-LAST (KW_FIND_LST)
**                  FIND-BY-ROWID (KW_FIND_BR)
** 014 ECF 20130309 Changed API for bufferCopy and bufferCompare; only wrapped data types are
**                  allowed for literal parameters, to prevent the explosion of the API. 
** 015 VMN 20130314 First parameter type changed from String to character for
**                  FIND-UNIQUE (KW_GET_UNI)
**                  FIND-FIRST (KW_FIND_1ST)
**                  FIND-LAST (KW_FIND_LST)
**                  BUFFER-COMPARE (KW_BUF_COMP): noLobs parameter type changed
** 016 CS  20130322 Moved the defaultBufferHandle method into BufferHandle interface.
** 017 CA  20130323 Changed the APIs for KW_BUF_DEL and KW_EMPTY_TT to return logical.
** 018 ECF 20130417 Moved initial() method to FieldInterface.
** 019 EVL 20130911 Adding legacy attributes and methods annotation.
*/

package com.goldencode.p2j.persist;

import com.goldencode.p2j.util.*;

/**
 * Defines the public API for services representing Progress builtin
 * functions, methods, attributes, and language statements, which operate on
 * record buffers.
 * <p>
 * Each DMO interface should contain a public, inner interface which extends
 * both this class and the enclosing DMO interface. This enables DMO proxies
 * used by business logic to invoke DMO-specific methods, as well as methods
 * which provide standard buffer services, without requiring a cast.
 */
public interface Buffer
extends WrappedResource,
        Errorable,
        BufferHandle,
        DatabaseInfo,
        ADMData,
        UniqueID,
        NamespaceURI,
        Dereferenceable
{
   /**
    * Get the record ID of the current buffer's current record, if any.  P2J
    * implementation of the RECID function.
    * <p>
    * This method will return a 32-bit recid which represents the internal
    * row ID of the given record.  If the actual row ID (which is implemented
    * as a wider integer) cannot fit due to overflow, an error condition will
    * be raised.
    *
    * @return  Record ID of the underlying data record. If no record currently
    *          backs the buffer associated with <code>dmo</code>, a recid
    *          initialized as unknown value is returned.
    * 
    * @throws  ErrorConditionException
    *          if the actual record ID is too wide to fit within a 32-bit
    *          integer.
    */
   @LegacyAttribute(name = "RECID")
   public recid recordID();
   
   /**
    * Get the row ID of the current buffer's current record, if any.  P2J
    * implementation of the ROWID function.
    *
    * @return  Row ID of the underlying data record.  If no record currently
    *          backs the buffer associated with <code>dmo</code>, a
    *          <code>rowid</code> initialized as unknown value is returned.
    */
   @LegacyAttribute(name = "ROWID")
   public rowid rowID();
   
   /**
    * Indicate whether a backing data record currently is available.
    *
    * @return  <code>true</code> if data is available, <code>false</code> if
    *          no record backs this buffer.
    */
   @LegacyAttribute(name = "AVAILABLE")
   public logical available();
   
   /**
    * Indicate whether a backing data record currently is available.
    *
    * @return  <code>true</code> if data is available, <code>false</code> if
    *          no record backs this buffer.
    */
   public boolean _available();
   
   /**
    * Checks if the record in the buffer has changed in database since the 
    * last <code>FIND CURRENT</code> or <code>GET CURRENT</code>.
    * P2J implementation of 4GL function <code>CURRENT-CHANGED</code>.
    *
    * @return  <code>true</code> if the database record differs from the
    *          value in memory buffer that was queried with the last
    *          <code>FIND</code> and <code>false</code> otherwise
    */
   public logical currentChanged();
   
   /**
    * Checks if the record in the buffer has changed in database since the 
    * last <code>FIND CURRENT</code> or <code>GET CURRENT</code>.
    * P2J implementation of 4GL function <code>CURRENT-CHANGED</code>.
    *
    * @return  <code>true</code> if the database record differs from the
    *          value in memory buffer that was queried with the last
    *          <code>FIND</code> and <code>false</code> otherwise
    */
   public boolean _currentChanged();
   
   /**
    * Indicate whether the buffer associated currently contains a record, and
    * whether that record is transient.  A record is transient if it has been
    * newly created and has not yet been saved to the database.
    *
    * @return  <code>true</code> if a record exists and is new, else
    *          <code>false</code>.
    */
   public logical newlyCreated();
   
   /**
    * Indicate whether the buffer associated currently contains a record, and
    * whether that record is transient.  A record is transient if it has been
    * newly created and has not yet been saved to the database.
    *
    * @return  <code>true</code> if a record exists and is new, else
    *          <code>false</code>.
    */
   public boolean _newlyCreated();
   
   /**
    * Indicate whether the last record retrieval attempt was denied because
    * the target record was locked.
    *
    * @return  <code>true</code> if the record was locked;  <code>false</code>
    *          if the attempt succeeded, or if this query has not yet
    *          attempted to retrieve a record.
    */
   @LegacyAttribute(name = "LOCKED")
   public logical locked();
   
   /**
    * Indicate whether the last record retrieval attempt was denied because
    * the target record was locked.
    *
    * @return  <code>true</code> if the record was locked;  <code>false</code>
    *          if the attempt succeeded, or if this query has not yet
    *          attempted to retrieve a record.
    */
   public boolean _locked();
   
   /**
    * Indicate whether the last record retrieval attempt failed because the
    * query criteria was ambiguous.
    *
    * @return  <code>true</code> if the query was ambiguous;
    *          <code>false</code> if the attempt succeeded, or if no attempt
    *          has yet been made.
    */
   @LegacyAttribute(name = "AMBIGUOUS")
   public logical ambiguous();
   
   /**
    * Indicate whether the last record retrieval attempt failed because the
    * query criteria was ambiguous.
    *
    * @return  <code>true</code> if the query was ambiguous;
    *          <code>false</code> if the attempt succeeded, or if no attempt
    *          has yet been made.
    */
   public boolean _ambiguous();
   
   /**
    * Create a new record in the buffer.
    *
    * @throws  ErrorConditionException
    *          if there is an error retrieving the next available primary key
    *          from the database or locking the record.
    */
   public void create();
   
   /**
    * Delete the record currently in the buffer, if any.
    *
    * @return  <code>true</code> if the delete was possible.
    *
    * @throws  ErrorConditionException
    *          if no record currently is loaded in the buffer, or if there is
    *          an error deleting the current record from the database.
    */
   @LegacyMethod(name = "BUFFER-DELETE")
   public logical deleteRecord();
   
   /**
    * Delete the record currently in the buffer, IF and ONLY IF the given
    * validator expression evaluates <code>true</code>.
    *
    * @param   validator
    *          The user-defined callback that is to be tested to determine if the
    *          delete should or should not occur.
    *
    * @return  <code>true</code> if the delete was possible.
    *
    * @throws  ErrorConditionException
    *          if no record currently is loaded in the buffer, or if there is
    *          an error deleting the current record from the database or if the
    *          validation expression fails.
    */
   @LegacyMethod(name = "BUFFER-DELETE")
   public logical deleteRecord(BufferValidator validator);
   
   /**
    * Delete all records associated with this buffer, which match the criteria
    * specified by the given where clause.  Note that this where clause must
    * not contain DMO alias qualifiers.
    * <p>
    * <b>NOTE:</b>  the current implementation must not be invoked for a DMO
    * which is backed by a permanent table;  only temp-table bulk delete is
    * supported at this time.
    * 
    * @param   where
    *          An HQL where clause snippet which defines the restriction
    *          criteria to apply to the delete.  All references to properties
    *          in a DMO must be unqualified.
    * @param   args
    *          Query substitution parameters required by the where clause.
    * 
    * @throws  ErrorConditionException
    *          if there is an error deleting the records.
    * @throws  UnsupportedOperationException
    *          if invoked on a buffer associated with a permanent table.
    */
   public void delete(String where, Object ...args);
   
   /**
    * Bulk delete all records associated with the buffer.
    * 
    * @return  <code>true</code> if the delete was possible.
    *
    * @throws  ErrorConditionException
    *          if an error occurs performing the bulk delete operation.
    */
   @LegacyMethod(name = "EMPTY-TEMP-TABLE")
   public logical deleteAll();
   
   /**
    * Aggregate method which averages the values of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void average(String column, NumberType num);
   
   /**
    * Aggregate method which averages the values of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * @param   where
    *          HQL where clause used to filter the records.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void average(String column, NumberType num, String where);
   
   /**
    * Aggregate method which averages the values of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * @param   where
    *          HQL where clause used to filter the records.
    * @param   args
    *          Query substitution arguments.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void average(String column, NumberType num, String where, Object...args);
   
   /**
    * Aggregate method which counts the rows in the table backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate. May be the wildcard
    *          character ("*").
    * @param   num
    *          Variable into which to store the aggregate result.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void count(String column, NumberType num);
   
   /**
    * Aggregate method which counts the rows in the table backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate. May be the wildcard
    *          character ("*").
    * @param   num
    *          Variable into which to store the aggregate result.
    * @param   where
    *          HQL where clause used to filter the records.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void count(String column, NumberType num, String where);
   
   /**
    * Aggregate method which counts the rows in the table backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate. May be the wildcard
    *          character ("*").
    * @param   num
    *          Variable into which to store the aggregate result.
    * @param   where
    *          HQL where clause used to filter the records.
    * @param   args
    *          Query substitution arguments.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void count(String column, NumberType num, String where, Object...args);
   
   /**
    * Aggregate method which finds the minimum value of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void minimum(String column, NumberType num);
   
   /**
    * Aggregate method which finds the minimum value of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * @param   where
    *          HQL where clause used to filter the records.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void minimum(String column, NumberType num, String where);
   
   /**
    * Aggregate method which finds the minimum value of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * @param   where
    *          HQL where clause used to filter the records.
    * @param   args
    *          Query substitution arguments.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void minimum(String column, NumberType num, String where, Object...args);
   
   /**
    * Aggregate method which finds the maximum value of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void maximum(String column, NumberType num);
   
   /**
    * Aggregate method which finds the maximum value of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * @param   where
    *          HQL where clause used to filter the records.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void maximum(String column, NumberType num, String where);
   
   /**
    * Aggregate method which finds the maximum value of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * @param   where
    *          HQL where clause used to filter the records.
    * @param   args
    *          Query substitution arguments.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void maximum(String column, NumberType num, String where, Object...args);
   
   /**
    * Aggregate method which sums the values of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void sum(String column, NumberType num);
   
   /**
    * Aggregate method which sums the values of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * @param   where
    *          HQL where clause used to filter the records.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void sum(String column, NumberType num, String where);
   
   /**
    * Aggregate method which sums the values of a single column across rows in the table
    * backing this buffer.
    * 
    * @param   column
    *          An expression indicating the name of the column to aggregate.
    * @param   num
    *          Variable into which to store the aggregate result.
    * @param   where
    *          HQL where clause used to filter the records.
    * @param   args
    *          Query substitution arguments.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the aggregate operation.
    */
   public void sum(String column, NumberType num, String where, Object...args);
   
   /**
    * Validate and release the record currently in the buffer, if any.
    *
    * @throws  ErrorConditionException
    *          if the buffer fails validation.
    */
   public void release();
   
   /**
    * Validate the record currently in the buffer, if any.
    *
    * @throws  ErrorConditionException
    *          if the buffer fails validation.
    */
   public void validate();
   
   /**
    * Open a read-only buffer scope (for trigger support).
    */
   public void openReadOnlyScope();

   /**
    * Conversion of BUFFER-FIELD() method (KW_BUF_FLD).
    *
    * Gets the handle to a particular buffer of a query or ProDataSet object.
    *
    * @param   fieldNumber
    *          An INTEGER expression representing the sequence number of the field in the buffer.
    *
    * @return  handle to a particular field in the buffer.
    */
   @LegacyMethod(name = "BUFFER-FIELD")
   public handle bufferField(int fieldNumber);

   /**
    * Conversion of BUFFER-FIELD() method (KW_BUF_FLD).
    *
    * Gets the handle to a particular buffer of a query or ProDataSet object.
    *
    * @param   fieldNumber
    *          An INTEGER expression representing the sequence number of the field in the buffer.
    *
    * @return  handle to a particular field in the buffer.
    */
   @LegacyMethod(name = "BUFFER-FIELD")
   public handle bufferField(NumberType fieldNumber);

   /**
    * Conversion of BUFFER-FIELD() method (KW_BUF_FLD).
    *
    * Gets the handle to a particular buffer of a query or ProDataSet object.
    *
    * @param   fieldName
    *          A string expression representing the name of the field in the buffer.
    *
    * @return  handle to a particular field in the buffer.
    */
   @LegacyMethod(name = "BUFFER-FIELD")
   public handle bufferField(String fieldName);
   
   /**
    * Conversion of BUFFER-FIELD() method (KW_BUF_FLD).
    *
    * Gets the handle to a particular buffer of a query or ProDataSet object.
    *
    * @param   fieldName
    *          A CHARACTER string expression representing the name of the field in the buffer.
    *
    * @return  handle to a particular field in the buffer.
    */
   @LegacyMethod(name = "BUFFER-FIELD")
   public handle bufferField(character fieldName);

   /**
    * Conversion of BUFFER-COPY() method (KW_BUF_COPY).
    *
    * This method copies any common fields, determined by name, data type,
    * and extent-matching, from the source buffer to the receiving buffer.
    * If there are fields in one buffer that do not exist in the other, they
    * are ignored. This method is used to accommodate temp-tables of joins.
    * 
    * @param   buffer
    *          The source buffer to copy from.
    *
    * @return  <code>true</code> if BUFFER-COPY() succeeds, else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COPY")
   public logical bufferCopy(Buffer buffer);

   /**
    * Conversion of BUFFER-COPY() method (KW_BUF_COPY).
    *
    * This method copies any common fields, determined by name, data type,
    * and extent-matching, from the source buffer to the receiving buffer.
    * If there are fields in one buffer that do not exist in the other, they
    * are ignored. This method is used to accommodate temp-tables of joins.
    * 
    * @param   buffer
    *          The source buffer to copy from.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the copy process.
    * 
    * @return  <code>true</code> if BUFFER-COPY() succeeds, else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COPY")
   public logical bufferCopy(Buffer buffer, character except);

   /**
    * Conversion of BUFFER-COPY() method (KW_BUF_COPY).
    *
    * This method copies any common fields, determined by name, data type,
    * and extent-matching, from the source buffer to the receiving buffer.
    * If there are fields in one buffer that do not exist in the other, they
    * are ignored. This method is used to accommodate temp-tables of joins.
    * 
    * @param   buffer
    *          The source buffer to copy from.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the copy process.
    * @param   pairs
    *          A comma-separated list of field-name pairs to be copied.  Each pair must contain 
    *          one field name from the source and one field name from the target.
    * 
    * @return  <code>true</code> if BUFFER-COPY() succeeds, else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COPY")
   public logical bufferCopy(Buffer buffer, character except, character pairs);

   /**
    * Conversion of BUFFER-COPY() method (KW_BUF_COPY).
    *
    * This method copies any common fields, determined by name, data type,
    * and extent-matching, from the source buffer to the receiving buffer.
    * If there are fields in one buffer that do not exist in the other, they
    * are ignored. This method is used to accommodate temp-tables of joins.
    * 
    * @param   buffer
    *          The source buffer to copy from.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the copy process.
    * @param   pairs
    *          A comma-separated list of field-name pairs to be copied.  Each pair must contain 
    *          one field name from the source and one field name from the target.
    * @param   noLobs
    *          If <code>TRUE</code>, BLOB and CLOB fields are ignored during the copy.
    * 
    * @return  <code>true</code> if BUFFER-COPY() succeeds, else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COPY")
   public logical bufferCopy(Buffer buffer, character except, character pairs, logical noLobs);

   /**
    * Conversion of BUFFER-COPY() method (KW_BUF_COPY).
    *
    * This method copies any common fields, determined by name, data type,
    * and extent-matching, from the source buffer to the receiving buffer.
    * If there are fields in one buffer that do not exist in the other, they
    * are ignored. This method is used to accommodate temp-tables of joins.
    * 
    * @param   bufHandle
    *          The handle of the source buffer to copy from.
    * 
    * @return  <code>true</code> if BUFFER-COPY() succeeds, else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COPY")
   public logical bufferCopy(handle bufHandle);

   /**
    * Conversion of BUFFER-COPY() method (KW_BUF_COPY).
    *
    * This method copies any common fields, determined by name, data type,
    * and extent-matching, from the source buffer to the receiving buffer.
    * If there are fields in one buffer that do not exist in the other, they
    * are ignored. This method is used to accommodate temp-tables of joins.
    * 
    * @param   bufHandle
    *          The handle of the source buffer to copy from.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the copy process.
    * 
    * @return  <code>true</code> if BUFFER-COPY() succeeds, else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COPY")
   public logical bufferCopy(handle bufHandle, character except);

   /**
    * Conversion of BUFFER-COPY() method (KW_BUF_COPY).
    *
    * This method copies any common fields, determined by name, data type,
    * and extent-matching, from the source buffer to the receiving buffer.
    * If there are fields in one buffer that do not exist in the other, they
    * are ignored. This method is used to accommodate temp-tables of joins.
    * 
    * @param   bufHandle
    *          The handle of the source buffer to copy from.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the copy process.
    * @param   pairs
    *          A comma-separated list of field-name pairs to be copied.  Each pair must contain 
    *          one field name from the source and one field name from the target.
    * 
    * @return  <code>true</code> if BUFFER-COPY() succeeds, else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COPY")
   public logical bufferCopy(handle bufHandle, character except, character pairs);

   /**
    * Conversion of BUFFER-COPY() method (KW_BUF_COPY).
    *
    * This method copies any common fields, determined by name, data type,
    * and extent-matching, from the source buffer to the receiving buffer.
    * If there are fields in one buffer that do not exist in the other, they
    * are ignored. This method is used to accommodate temp-tables of joins.
    * 
    * @param   bufHandle
    *          The handle of the source buffer to copy from.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the copy process.
    * @param   pairs
    *          A comma-separated list of field-name pairs to be copied.  Each pair must contain 
    *          one field name from the source and one field name from the target.
    * @param   noLobs
    *          If <code>TRUE</code>, BLOB and CLOB fields are ignored during the copy.
    * 
    * @return  <code>true</code> if BUFFER-COPY() succeeds, else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COPY")
   public logical bufferCopy(handle bufHandle, character except, character pairs, logical noLobs);

   /**
    * Conversion of BUFFER-COMPARE() method (KW_BUF_COMP).
    *
    * This method does a rough compare of any common fields, determined by
    * name, data type, and extent-matching, between the source buffer and the
    * target buffer. The resulting logical value is either TRUE or FALSE as
    * a whole. A single field that does not compare causes the entire buffer
    * to return FALSE. If there are fields in one buffer that do not exist in
    * the other, they are ignored.
    *
    * @param   buffer
    *          Buffer with which to compare.
    *
    * @return  <code>true</code> if BUFFER-COMPARE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COMPARE")
   public logical bufferCompare(Buffer buffer);

   /**
    * Conversion of BUFFER-COMPARE() method (KW_BUF_COMP).
    *
    * This method does a rough compare of any common fields, determined by
    * name, data type, and extent-matching, between the source buffer and the
    * target buffer. The resulting logical value is either TRUE or FALSE as
    * a whole. A single field that does not compare causes the entire buffer
    * to return FALSE. If there are fields in one buffer that do not exist in
    * the other, they are ignored.
    *
    * @param   buffer
    *          Buffer with which to compare.
    * @param   mode
    *          The text "binary" or "case-sensitive", to provide that type of comparison.
    *
    * @return  <code>true</code> if BUFFER-COMPARE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COMPARE")
   public logical bufferCompare(Buffer buffer, character mode);

   /**
    * Conversion of BUFFER-COMPARE() method (KW_BUF_COMP).
    *
    * This method does a rough compare of any common fields, determined by
    * name, data type, and extent-matching, between the source buffer and the
    * target buffer. The resulting logical value is either TRUE or FALSE as
    * a whole. A single field that does not compare causes the entire buffer
    * to return FALSE. If there are fields in one buffer that do not exist in
    * the other, they are ignored.
    *
    * @param   buffer
    *          Buffer with which to compare.
    * @param   mode
    *          The text "binary" or "case-sensitive", to provide that type of comparison.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the compare process.
    *
    * @return  <code>true</code> if BUFFER-COMPARE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COMPARE")
   public logical bufferCompare(Buffer buffer, character mode, character except);

   /**
    * Conversion of BUFFER-COMPARE() method (KW_BUF_COMP).
    *
    * This method does a rough compare of any common fields, determined by
    * name, data type, and extent-matching, between the source buffer and the
    * target buffer. The resulting logical value is either TRUE or FALSE as
    * a whole. A single field that does not compare causes the entire buffer
    * to return FALSE. If there are fields in one buffer that do not exist in
    * the other, they are ignored.
    *
    * @param   buffer
    *          Buffer with which to compare.
    * @param   mode
    *          The text "binary" or "case-sensitive", to provide that type of comparison.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the compare process.
    * @param   pairs
    *          A comma-separated list of field-name pairs to be compared.  Each pair must contain
    *          one field name from the source and one field name from the target.
    *
    * @return  <code>true</code> if BUFFER-COMPARE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COMPARE")
   public logical bufferCompare(Buffer buffer, character mode, character except, character pairs);

   /**
    * Conversion of BUFFER-COMPARE() method (KW_BUF_COMP).
    *
    * This method does a rough compare of any common fields, determined by
    * name, data type, and extent-matching, between the source buffer and the
    * target buffer. The resulting logical value is either TRUE or FALSE as
    * a whole. A single field that does not compare causes the entire buffer
    * to return FALSE. If there are fields in one buffer that do not exist in
    * the other, they are ignored.
    *
    * @param   buffer
    *          Buffer with which to compare.
    * @param   mode
    *          The text "binary" or "case-sensitive", to provide that type of comparison.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the compare process.
    * @param   pairs
    *          A comma-separated list of field-name pairs to be compared.  Each pair must contain
    *          one field name from the source and one field name from the target.
    * @param   noLobs
    *          If <code>TRUE</code>, BLOB and CLOB fields are ignored during the copy.
    *
    * @return  <code>true</code> if BUFFER-COMPARE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COMPARE")
   public logical bufferCompare(Buffer    buffer,
                                character mode,
                                character except,
                                character pairs,
                                logical   noLobs);

   /**
    * Conversion of BUFFER-COMPARE() method (KW_BUF_COMP).
    *
    * This method does a rough compare of any common fields, determined by
    * name, data type, and extent-matching, between the source buffer and the
    * target buffer. The resulting logical value is either TRUE or FALSE as
    * a whole. A single field that does not compare causes the entire buffer
    * to return FALSE. If there are fields in one buffer that do not exist in
    * the other, they are ignored.
    *
    * @param   bufHandle
    *          Handle of buffer with which to compare.
    *
    * @return  <code>true</code> if BUFFER-COMPARE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COMPARE")
   public logical bufferCompare(handle bufHandle);

   /**
    * Conversion of BUFFER-COMPARE() method (KW_BUF_COMP).
    *
    * This method does a rough compare of any common fields, determined by
    * name, data type, and extent-matching, between the source buffer and the
    * target buffer. The resulting logical value is either TRUE or FALSE as
    * a whole. A single field that does not compare causes the entire buffer
    * to return FALSE. If there are fields in one buffer that do not exist in
    * the other, they are ignored.
    *
    * @param   bufHandle
    *          Handle of buffer with which to compare.
    * @param   mode
    *          The text "binary" or "case-sensitive", to provide that type of comparison.
    *
    * @return  <code>true</code> if BUFFER-COMPARE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COMPARE")
   public logical bufferCompare(handle bufHandle, character mode);

   /**
    * Conversion of BUFFER-COMPARE() method (KW_BUF_COMP).
    *
    * This method does a rough compare of any common fields, determined by
    * name, data type, and extent-matching, between the source buffer and the
    * target buffer. The resulting logical value is either TRUE or FALSE as
    * a whole. A single field that does not compare causes the entire buffer
    * to return FALSE. If there are fields in one buffer that do not exist in
    * the other, they are ignored.
    *
    * @param   bufHandle
    *          Handle of buffer with which to compare.
    * @param   mode
    *          The text "binary" or "case-sensitive", to provide that type of comparison.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the compare process.
    *
    * @return  <code>true</code> if BUFFER-COMPARE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COMPARE")
   public logical bufferCompare(handle bufHandle, character mode, character except);

   /**
    * Conversion of BUFFER-COMPARE() method (KW_BUF_COMP).
    *
    * This method does a rough compare of any common fields, determined by
    * name, data type, and extent-matching, between the source buffer and the
    * target buffer. The resulting logical value is either TRUE or FALSE as
    * a whole. A single field that does not compare causes the entire buffer
    * to return FALSE. If there are fields in one buffer that do not exist in
    * the other, they are ignored.
    *
    * @param   bufHandle
    *          Handle of buffer with which to compare.
    * @param   mode
    *          The text "binary" or "case-sensitive", to provide that type of comparison.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the compare process.
    * @param   pairs
    *          A comma-separated list of field-name pairs to be compared.  Each pair must contain
    *          one field name from the source and one field name from the target.
    *
    * @return  <code>true</code> if BUFFER-COMPARE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COMPARE")
   public logical bufferCompare(handle bufHandle,
                                character mode,
                                character except,
                                character pairs);

   /**
    * Conversion of BUFFER-COMPARE() method (KW_BUF_COMP).
    *
    * This method does a rough compare of any common fields, determined by
    * name, data type, and extent-matching, between the source buffer and the
    * target buffer. The resulting logical value is either TRUE or FALSE as
    * a whole. A single field that does not compare causes the entire buffer
    * to return FALSE. If there are fields in one buffer that do not exist in
    * the other, they are ignored.
    *
    * @param   bufHandle
    *          Handle of buffer with which to compare.
    * @param   mode
    *          The text "binary" or "case-sensitive", to provide that type of comparison.
    * @param   except
    *          A comma-separated list of fields that will be ignored in the compare process.
    * @param   pairs
    *          A comma-separated list of field-name pairs to be compared.  Each pair must contain
    *          one field name from the source and one field name from the target.
    * @param   noLobs
    *          If <code>TRUE</code>, BLOB and CLOB fields are ignored during the copy.
    *
    * @return  <code>true</code> if BUFFER-COMPARE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-COMPARE")
   public logical bufferCompare(handle    bufHandle,
                                character mode,
                                character except,
                                character pairs,
                                logical   noLobs);

   /**
    * Conversion of BUFFER-CREATE() method (KW_BUF_CREA).
    *
    * Creates a record, sets fields to their default values, and moves a copy of the record into 
    * the buffer.
    *
    * @return  <code>true</code> if successful
    */
   @LegacyMethod(name = "BUFFER-CREATE")
   public logical bufferCreate();

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-FIRST")
   public logical findFirst();

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-FIRST")
   public logical findFirst(character whereClause);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lockType
    *          Lock type
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-FIRST")
   public logical findFirst(character whereClause, LockType lockType);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-FIRST")
   public logical findFirst(character whereClause, long lock);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-FIRST")
   public logical findFirst(character whereClause, NumberType lock);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-FIRST")
   public logical findFirst(character whereClause, long lock, long nowait);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-FIRST")
   public logical findFirst(character whereClause, NumberType lock, long nowait);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-FIRST")
   public logical findFirst(character whereClause, long lock, NumberType nowait);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-FIRST")
   public logical findFirst(character whereClause, NumberType lock, NumberType nowait);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-LAST")
   public logical findLast();

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-LAST")
   public logical findLast(character whereClause);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lockType
    *          Lock type
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-LAST")
   public logical findLast(character whereClause, LockType lockType);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-LAST")
   public logical findLast(character whereClause, long lock);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-LAST")
   public logical findLast(character whereClause, NumberType lock);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-LAST")
   public logical findLast(character whereClause, long lock, long nowait);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-LAST")
   public logical findLast(character whereClause, NumberType lock, long nowait);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-LAST")
   public logical findLast(character whereClause, long lock, NumberType nowait);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-LAST")
   public logical findLast(character whereClause, NumberType lock, NumberType nowait);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-UNIQUE")
   public logical findUnique();

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-UNIQUE")
   public logical findUnique(character whereClause);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lockType
    *          Lock type
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-UNIQUE")
   public logical findUnique(character whereClause, LockType lockType);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-UNIQUE")
   public logical findUnique(character whereClause, long lock);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-UNIQUE")
   public logical findUnique(character whereClause, NumberType lock);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-UNIQUE")
   public logical findUnique(character whereClause, long lock, long nowait);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-UNIQUE")
   public logical findUnique(character whereClause, NumberType lock, long nowait);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-UNIQUE")
   public logical findUnique(character whereClause, long lock, NumberType nowait);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "FIND-UNIQUE")
   public logical findUnique(character whereClause, NumberType lock, NumberType nowait);

   /**
    * Conversion of BUFFER-RELEASE() method (KW_BUF_REL).
    *
    * Releases a record from a buffer object. The BUFFER-RELEASE method
    * corresponds to the RELEASE statement.
    *
    * @return  <code>true</code> if BUFFER-RELEASE() succeeds,
    *          else <code>false</code>.
    */
   @LegacyMethod(name = "BUFFER-RELEASE")
   public logical bufferRelease();

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   @LegacyMethod(name = "FIND-BY-ROWID")
   public logical findByRowID(rowid id);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lockType
    *          Lock type
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   @LegacyMethod(name = "FIND-BY-ROWID")
   public logical findByRowID(rowid id, LockType lockType);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   @LegacyMethod(name = "FIND-BY-ROWID")
   public logical findByRowID(rowid id, long lock);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   @LegacyMethod(name = "FIND-BY-ROWID")
   public logical findByRowID(rowid id, NumberType lock);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   @LegacyMethod(name = "FIND-BY-ROWID")
   public logical findByRowID(rowid id, long lock, long nowait);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   @LegacyMethod(name = "FIND-BY-ROWID")
   public logical findByRowID(rowid id, NumberType lock, long nowait);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   @LegacyMethod(name = "FIND-BY-ROWID")
   public logical findByRowID(rowid id, long lock, NumberType nowait);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   @LegacyMethod(name = "FIND-BY-ROWID")
   public logical findByRowID(rowid id, NumberType lock, NumberType nowait);

   /**
    * Conversion of NUM-FIELDS attribute (KW_NUM_FLD).
    *
    * Getter of NUM-FIELDS attribute.
    *
    * @return  current value of NUM-FIELDS attribute.
    */
   @LegacyAttribute(name = "NUM-FIELDS")
   public integer numFields();

   /**
    * Disable the triggers associated with the FIND event for the buffer's table.
    */
   @LegacyMethod(name = "DISABLE-DUMP-TRIGGERS")
   public void disableDumpTriggers();

   /**
    * Disable the triggers associated with the CREATE, WRITE, DELETE and ASSIGN events for the
    * buffer's table.
    *
    * @param allowReplication
    *        If <code>true</code> then only CREATE, DELETE, ASSIGN and WRITE triggers are
    *        disabled, and not REPLICATION-CREATE, REPLICATION-DELETE and REPLICATION-WRITE
    *        triggers.
    */
   @LegacyMethod(name = "DISABLE-LOAD-TRIGGERS")
   public void disableLoadTriggers(boolean allowReplication);

   /**
    * Conversion of CURRENT-ITERATION attribute (KW_CUR_ITER).
    *
    * Getter of CURRENT-ITERATION attribute.
    *
    * @return current value of CURRENT-ITERATION attribute.
    */
   @LegacyAttribute(name = "CURRENT-ITERATION")
   public integer currentIteration();

   /**
    * Conversion of TABLE-HANDLE attribute (KW_TAB_HAND).
    *
    * Gets the handle of a temp-table object, if any, associated with the buffer object.
    *
    * @return  the handle of a temp-table object.
    */
   @LegacyAttribute(name = "TABLE-HANDLE")
   public handle tableHandle();
}
