/*
** Module   : SOAPHeaderEntry.java
** Abstract : Interface defining attributes/methods for SOAP-HEADER-ENTRYREF resource.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130216 Created initial version.
** 002 CA  20130221 Added ADM-DATA, UNIQUE-ID, NAMESPACE-URI and LOCAL-NAME support.
** 003 EVL 20130911 Adding legacy attributes and methods annotation.
*/

package com.goldencode.p2j.util;

/**
 * Interface describing the features for SOAP-HEADER-ENTRY resource:
 * <ul>
 *    <li>{@link #getActor ACTOR attribute}</li>
 *    <li>{@link #isMustUnderstand MUST-UNDERSTAND attribute}</li>
 *    <li>{@link #deleteHeaderEntry DELETE-HEADER-ENTRY method}</li>
 *    <li>{@link #setActor SET-ACTOR method}</li>
 *    <li>{@link #setMustUnderstand SET-MUST-UNDERSTAND method}</li>
 *    <li>{@link #setNode SET-NODE method}</li>
 *    <li>{@link #setSerialized SET-SERIALIZED method}</li>
 * </ul>
 */
public interface SOAPHeaderEntry
extends LocalName,
        ADMData,
        UniqueID,
        NamespaceURI
{
   /**
    * Returns the value of the actor attribute for the SOAP-HEADER-ENTRYREF object as a URL. 
    * Identifies the recipient of a header element.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "ACTOR")
   public character getActor();

   /**
    * Indicates whether a SOAP-HEADER-ENTRYREF object is mandatory (TRUE) or optional (FALSE) for
    * the recipient to process.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "MUST-UNDERSTAND")
   public logical isMustUnderstand();

   /**
    * Deletes the XML underlying a SOAP-HEADER-ENTRYREF object, without deleting the object.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "DELETE-HEADER-ENTRY")
   public logical deleteHeaderEntry();

   /**
    * Sets the ACTOR attribute for this SOAP-HEADER-ENTRY.
    *
    * @param    uri
    *           The URI of the SOAP actor.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-ACTOR")
   public logical setActor(character uri);

   /**
    * Sets the ACTOR attribute for this SOAP-HEADER-ENTRY.
    *
    * @param    uri
    *           The URI of the SOAP actor.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-ACTOR")
   public logical setActor(String uri);

   /**
    * Sets the MUST-UNDERSTAND attribute for this SOAP-HEADER-ENTRY.
    *
    * @param    val
    *           The new value of this attribute.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-MUST-UNDERSTAND")
   public logical setMustUnderstand(logical val);

   /**
    * Sets the MUST-UNDERSTAND attribute for this SOAP-HEADER-ENTRY.
    *
    * @param    val
    *           The new value of this attribute.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-MUST-UNDERSTAND")
   public logical setMustUnderstand(boolean val);

   /**
    * Replaces the underlying XML of a SOAP-HEADER-ENTRYREF object with the DOM XML sub-tree 
    * rooted by the X-NODEREF object.
    *
    * @param    xNoderef
    *           A X-NODEREF variable that refers to the XML that will be set for the 
    *           SOAP-HEADER-ENTRYREF object.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-NODE")
   public logical setNode(handle xNoderef);

   /**
    * Sets the SOAP-header-entryref object's underlying XML from serialized XML.
    * 
    * @param    xml
    *           A LONGCHAR variable that contains the XML that will become this 
    *           SOAP-HEADER-ENTRYREF object. This parameter can be only a longchar, 4GL doesn't
    *           allow character variables.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "SET-SERIALIZED")
   public logical setSerialized(longchar xml);
}